package com.jianshui.platform.utils;

import com.jianshui.common.constant.InvoiceCommonConstants;
import org.springframework.util.StringUtils;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @Author: kane
 * @Description: 批量下载工具类
 * @CreateTime: 2023-06-09  15:46
 * @Version: 1.0
 **/
public class BatchDownloadUtils {
    /**
     * 功能描述: 批量下载
     * @param status 文件类型0-pfd 1-ofd
     * @param urls
     * @return : void
     */
    public static void downloadAndZipFiles(List<String> urls, String status, HttpServletResponse response) {
        Date now = new Date();
        SimpleDateFormat format = new SimpleDateFormat(InvoiceCommonConstants.DATEFORMAT);
        String name = format.format(now);
        String zipFileName = name + ".zip";
        OutputStream ops = null;
        ZipOutputStream zos = null;
        InputStream inputStream = null;
        try {
            response.setHeader("Content-Disposition", "attachment;filename=" + zipFileName);
            response.setCharacterEncoding("utf-8");
            ops = response.getOutputStream();
            zos = new ZipOutputStream(ops);
            for (String url : urls) {
                if (StringUtils.hasText(url)) {
                    URL downloadUrl = new URL(url);
                    HttpURLConnection httpConn = (HttpURLConnection) downloadUrl.openConnection();
                    int responseCode = httpConn.getResponseCode();
                    if (responseCode == HttpURLConnection.HTTP_OK) {
                        String fileName = getFileName(httpConn, status);
                        inputStream = httpConn.getInputStream();
                        ZipEntry zipEntry = new ZipEntry(fileName);
                        zos.putNextEntry(zipEntry);
                        int bytesRead = -1;
                        byte[] buffer = new byte[4096];
                        while ((bytesRead = inputStream.read(buffer)) != -1) {
                            zos.write(buffer, 0, bytesRead);
                        }
                    }
                    httpConn.disconnect();
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }finally {
            if (inputStream != null){
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (zos != null){
                try {
                    zos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (ops != null){
                try {
                    ops.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 功能描述: 获取下载后的文件名
     * @param httpConn
     * @return : java.lang.String
     */
    private static String getFileName(HttpURLConnection httpConn, String status) {
        String fileName = "";
        String disposition = httpConn.getHeaderField("Content-Disposition");
        Date date = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(InvoiceCommonConstants.DATEFORMAT);
        if (disposition != null) {
            int index = disposition.indexOf("filename=");
            if (index > 0) {
                fileName = disposition.substring(index + 10, disposition.length() - 1);
                if ("0".equals(status)) {
                    String name = simpleDateFormat.format(date);
                    fileName = name + ".pdf";
                } else {
                    String name = simpleDateFormat.format(date);
                    fileName = name + ".ofd";
                }
            }
        }
        if (StringUtils.isEmpty(fileName)) {
            fileName = httpConn.getURL().getFile().substring(httpConn.getURL().getFile().lastIndexOf("/") + 1);
            if ("0".equals(status)) {
                String name = simpleDateFormat.format(date);
                fileName = name + ".pdf";
            } else {
                String name = simpleDateFormat.format(date);
                fileName = name + ".ofd";
            }
        }
        return fileName;
    }
}
